import std/json, std/logging
import docopt

import private/cliconstants
import private/libvlc
import wdiwtlt/config
import wdiwtlt/db
import wdiwtlt/library

type WdiwtltContext = ref object
  cfg: WdiwtltConfig
  player: VlcMediaPlayer
  vlc: LibVlcInstance
  library: WdiwtltLibrary

proc initContext(cfg: WdiwtltConfig): WdiwtltContext =
  let vlc = newVlc()
  return WdiwtltContext(
    cfg: cfg,
    player: vlc.newMediaPlayer,
    vlc: vlc,
    library: initLibrary(cfg.libraryPath, cfg.dbPath))

proc release(ctx: WdiwtltContext) =
  if not ctx.player.isNil: ctx.player.release
  if not ctx.vlc.isNil: ctx.vlc.release

when isMainModule:
 var ctx: WdiwtltContext
 try:

  let consoleLogger = newConsoleLogger(
    #levelThreshold=lvlInfo,
    levelThreshold=lvlDebug,
    fmtStr="pit - $levelname: ")
  logging.addHandler(consoleLogger)

  let args = docopt(USAGE, version = WDIWTLT_VERSION)

  let cfg = loadConfig(args)

  if args["init-library"]:
    info "Initializing a new library database at '" & cfg.dbPath & "'"
    let newDb = initDb(cfg.dbPath)
    newDb.persist

    info "Writing configuration to '" & cfg.cfgPath & "'"
    cfg.cfgPath.writeFile(pretty(%*{
      "dbPath": cfg.dbPath,
      "libraryPath": cfg.libraryPath
    }))

    quit(0)

  ctx = cfg.initContext

  let media = ctx.vlc.mediaFromPath("file:///C:/Users/Jonathan Bernard/programs/vlc-3.0.18/2022-08-06.wav")
  ctx.player.setMedia(media)
  debug "Initialized LibVLC instance. Playing..."
  discard ctx.player.play

  discard stdin.readline
  ctx.player.stop
  media.release

 except:
  fatal getCurrentExceptionMsg()
  quit(QuitFailure)

 finally:
  if not ctx.isNil: ctx.release
  debug "Released LibVLC instance."
